/*==============================================================================
 The Fraclet Programming Model - Fractal Component Model (fractal@objectweb.org)
 Copyright (C) 2002-2006 INRIA Futurs / LIFL
 Copyright (C) 2006-2007 Universitetet i Oslo

 This library is free software; you can redistribute it and/or modify it under 
 the terms of the GNU Lesser General Public License as published by the Free 
 Software Foundation; either version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful, but WITHOUT ANY 
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License along 
 with this library; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 Initial developer(s): Romain Rouvoy
 Contributor(s): 
 =============================================================================*/
package org.objectweb.fractal.spoonlet.component;

import static org.objectweb.fractal.fraclet.types.Constants.EMPTY;

import java.lang.annotation.Annotation;

import org.objectweb.fractal.fraclet.annotations.Component;

import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtField;
import spoon.reflect.declaration.CtSimpleType;
import spoon.reflect.declaration.CtType;
import spoon.reflect.reference.CtTypeReference;

/**
 * Convenience class providing methods for handling the component concepts.
 * 
 * @author Romain Rouvoy (Initial Developer)
 * @version $Revision$
 */
public class ComponentHelper {
    /**
     * Checks if the component supports a given controller.
     * 
     * @param comp the component processed.
     * @param ctrl the controller requested.
     * @param all checks recursively when true.
     * @return true if the component already implements the requested
     *         controller.
     */
    public static final boolean hasController(CtType<?> comp,
            CtTypeReference<?> ctrl, boolean all) {
        if (all)
            return comp.isSubtypeOf(ctrl);
        for (CtTypeReference<?> itf : comp.getSuperInterfaces())
            if (itf.isSubtypeOf(ctrl))
                return true;
        return false;
    }

    /**
     * Checks if the component declared a given field annotation.
     * 
     * @param comp the component processed.
     * @param ctrl the controller requested.
     * @param all checks recursively when true.
     * @return true if the component already implements the requested
     *         controller.
     */
    public static final boolean hasFieldAnnotation(CtClass<?> comp,
            Class<? extends Annotation> ann, boolean all) {
        if (comp.getActualClass().equals(Object.class))
            return false;
        for (CtField<?> f : comp.getFields())
            if (f.getAnnotation(ann) != null)
                return true;
        if (all && comp.getSuperclass() != null)
            return hasFieldAnnotation((CtClass<?>) comp.getSuperclass()
                    .getDeclaration(), ann, all);
        return false;
    }

    /**
     * Provides the fully-qualified name of the component.
     * 
     * @param comp the component processed.
     * @param ann the annotation <code>Component</code> processed.
     * @return the fully-qualified name of the component.
     */
    public static final String componentFullname(CtSimpleType<?> comp,
            Component ann) {
        return EMPTY.equals(ann.name()) ? comp.getQualifiedName() : ann.name();
    }
}