package org.objectweb.fractal.adl.spoonlet.composite;

import static java.util.logging.Level.FINER;
import static java.util.logging.Logger.getLogger;
import static org.objectweb.fractal.adl.spoonlet.composite.SubComponentProcessor.CLIENT;
import static org.objectweb.fractal.adl.spoonlet.composite.DefinitionGenerator.composite;
import static org.objectweb.fractal.adl.spoonlet.definition.Arguments.asArgument;
import static org.objectweb.fractal.spoonlet.component.InterfaceHelper.interfaceName;
import static org.objectweb.fractal.spoonlet.utils.Constants.LOG;

import java.util.logging.Logger;

import org.objectweb.fractal.document.Element;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Requires;

import spoon.processing.AbstractAnnotationProcessor;
import spoon.reflect.declaration.CtClass;
import spoon.reflect.declaration.CtInterface;
import spoon.reflect.reference.CtTypeReference;

/**
 * @author Romain Rouvoy (Initial Developer)
 * @version $Revision$
 */
public class ExportInterfaceProcessor extends
        AbstractAnnotationProcessor<Component, CtClass<?>> implements
        BindingTags {
    private final Logger log = getLogger("spoonlet.fractaladl.binding");

    /* (non-Javadoc)
     * @see spoon.processing.AbstractProcessor#init()
     */
    @Override
    public void init() {
        addProcessedAnnotationType(Component.class);
        super.init();
    }

    /* (non-Javadoc)
     * @see spoon.processing.AbstractAnnotationProcessor#inferConsumedAnnotationType()
     */
    @Override
    public boolean inferConsumedAnnotationType() {
        return false;
    }


    /* (non-Javadoc)
     * @see spoon.processing.AnnotationProcessor#process(java.lang.annotation.Annotation, spoon.reflect.declaration.CtElement)
     */
    public void process(Component a, CtClass<?> cls) {
        if (cls.getAnnotatedChildren(Requires.class).size() == 0)
            return;
        if (LOG && log.isLoggable(FINER))
            log.finer("Fractal ADL - Processing class "
                    + cls.getQualifiedName() + "...");
        Element root = composite().definition(cls);
        for (Element e : root.getChilds())
            if (BINDING_TAG.equals(e.getName()))
                return;
        for (CtTypeReference<?> ref : cls.getSuperInterfaces()) {
            try {
                Interface ann = ref.getAnnotation(Interface.class);
                if (ann == null)
                    continue;
                Element binding = new Element(BINDING_TAG);
                String itfName = interfaceName(ann, (CtInterface<?>) ref
                        .getDeclaration());
                binding.setAttribute(BIND_CLIENT, "this." + itfName);
                binding.setAttribute(BIND_SERVER, asArgument.apply(CLIENT) + "."
                        + itfName);
                root.getChilds().add(binding);

            } catch (Exception e) {
            }
        }
        for (Interface itf : cls.getAnnotation(Component.class).provides()) {
            Element binding = new Element(BINDING_TAG);
            String itfName = interfaceName(itf, (CtInterface<?>) getFactory()
                    .Type().get(itf.signature()));
            binding.setAttribute(BIND_CLIENT, "this." + itfName);
            binding.setAttribute(BIND_SERVER, asArgument.apply(CLIENT) + "."
                    + itfName);
            root.getChilds().add(binding);
        }
    }
}
