/*==============================================================================
 The Fraclet Programming Model - Fractal Component Model (fractal@objectweb.org)
 Copyright (C) 2002-2006 INRIA Futurs / LIFL
 Copyright (C) 2006-2007 Universitetet i Oslo

 This library is free software; you can redistribute it and/or modify it under 
 the terms of the GNU Lesser General Public License as published by the Free 
 Software Foundation; either version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful, but WITHOUT ANY 
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License along 
 with this library; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

 Initial developer(s): Romain Rouvoy
 Contributor(s): 
 =============================================================================*/
package org.objectweb.fractal.document;

import static com.google.common.base.Objects.nonNull;
import static com.google.common.collect.Maps.newHashMap;

import java.util.Map;

import spoon.reflect.declaration.CtType;

/**
 * Fractal ADL description factory.
 * 
 * @author Romain Rouvoy (Initial Developer)
 * @version $Revision$
 */
public class Factory {
    /** List of descriptions already created. */
    private Map<CtType<?>, Document> documents = newHashMap();

    /**
     * Provides the reference of the Fractal ADL description associated to the
     * component. Creates a new definition if no description has been already
     * created.
     * 
     * @param cls the component processed.
     * @return the reference of the Fractal ADL description associated to this
     *         component.
     */
    public Document document(CtType<?> cls) {
        if (this.documents.get(cls) == null)
            this.documents.put(cls, new Document(cls.getQualifiedName()));
        return this.documents.get(cls);
    }

    /**
     * Provides the reference of the Fractal ADL definition associated to the
     * component.
     * 
     * @param cls the component processed.
     * @return the reference of the Fractal ADL definition associated to this
     *         component.
     * @throws RuntimeException exception thrown if the root definition
     */
    public Element definition(CtType<?> cls) {
        return nonNull(document(cls).getRoot(), "Definition " + cls.getQualifiedName()
                + " does not exist");
    }

    /**
     * Applies a visitor on each Fractal ADL description created.
     * 
     * @param v the visitor to apply.
     */
    public void accept(DocumentVisitor v) {
        for (Document d : documents.values())
            d.accept(v);
    }
}
